import os
import json
import pytest
from datetime import datetime
from typing import List
from coinmetrics.data_exporter import CoinMetricsDataExporter


CM_API_KEY = os.environ.get("CM_API_KEY")
data_exporter = CoinMetricsDataExporter(api_key=str(CM_API_KEY))
cm_api_key_set = CM_API_KEY is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"


@pytest.fixture()
def setup_function() -> None:
    folders_to_check_and_delete = [
        "data",
        "market-trades-spot",
        "market-trades-future",
        "market-candles-future",
        "market-candles-spot",
    ]
    print(
        f"Deleting any folders that test folders that exist before tests are run: {folders_to_check_and_delete}"
    )
    for dir in folders_to_check_and_delete:
        if os.path.exists(dir):
            os.rmdir(dir)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def files_downloaded_test_helper_start(list_of_expected_files: List[str]) -> None:
    """
    This is a helper function that test that the provided files do not exist at the time the tests start
    """
    for file in list_of_expected_files:
        if os.path.exists(file):
            print(
                f"The file {file} is not supposed to exist before this test is run, failing"
            )
            assert not (os.path.exists(file))


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def files_downloaded_test_helper_end(list_of_file_locations: List[str]) -> None:
    """
    This is a helper function that tests that the expected files were created and then deletes them to clean up
    """
    try:
        for file in list_of_file_locations:
            assert os.path.exists(file)
    except Exception as e:
        print("Exceptions found, removing files then raising exceptions")
        for file in list_of_file_locations:
            if os.path.exists(file):
                os.remove(file)
        raise e


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_export_market_candles_5m() -> None:
    list_of_expected_files = [
        "market-candles-future-5m/binance/2019-10-10.json.gz",
        "market-candles-future-5m/binance/2019-10-11.json.gz",
        "market-candles-future-5m/binance/2019-10-12.json.gz",
    ]
    files_downloaded_test_helper_start(list_of_expected_files)
    start_date = datetime(2019, 10, 10)
    end_date = datetime(2019, 10, 12)
    exchange = "binance"
    data_exporter.export_market_candles_future_data(
        frequency="5m",
        start_date=start_date,
        end_date=end_date,
        exchanges=exchange,
        output_format="csv",
    )


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_export_market_quotes_future_one_asset() -> None:
    list_of_expected_files = [
        "market-quotes-future/Binance/1INCHUSDT/2022-02-19.json.gz",
        "market-quotes-future/Binance/1INCHUSDT/2022-02-20.json.gz",
        "market-quotes-future/Binance/1INCHUSDT/2022-02-21.json.gz",
    ]
    files_downloaded_test_helper_start(list_of_expected_files)
    start_date = datetime(2022, 2, 19)
    end_date = datetime(2022, 2, 21)
    exchanges = ["Binance"]
    asset_pairs = ["1INCHUSDT"]
    data_exporter.export_market_quotes_future_data(
        start_date=start_date,
        end_date=end_date,
        asset_pairs=asset_pairs,
        exchanges=exchanges,
        output_format="json.gz",
    )
    files_downloaded_test_helper_end(list_of_expected_files)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_files_to_download_mquote() -> None:
    list_of_expected_files = [
        "market-quotes-spot/Binance/1INCHBTC/2022-02-19.json.gz",
        "market-quotes-spot/Binance/1INCHBTC/2022-02-20.json.gz",
        "market-quotes-spot/Binance/1INCHBTC/2022-02-21.json.gz",
    ]
    start_date = datetime(2022, 2, 19)
    end_date = datetime(2022, 2, 21)
    exchanges = ["Binance"]
    asset_pairs = ["1INCHBTC"]
    actual_files = data_exporter._get_list_files_to_download_from_ff_server(
        route_url="market-quotes-spot",
        assets_pairs=asset_pairs,
        exchanges=exchanges,
        start_date=start_date,
        end_date=end_date,
    )
    assert actual_files == list_of_expected_files


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_files_to_download_mquote_future() -> None:
    list_of_expected_files = [
        "market-quotes-future/Binance/1INCHUSDT/2022-02-19.json.gz",
        "market-quotes-future/Binance/1INCHUSDT/2022-02-20.json.gz",
        "market-quotes-future/Binance/1INCHUSDT/2022-02-21.json.gz",
    ]
    start_date = datetime(2022, 2, 19)
    end_date = datetime(2022, 2, 21)
    exchanges = ["Binance"]
    asset_pairs = ["1INCHUSDT"]
    actual_files = data_exporter._get_list_files_to_download_from_ff_server(
        route_url="market-quotes-future",
        assets_pairs=asset_pairs,
        exchanges=exchanges,
        start_date=start_date,
        end_date=end_date,
    )
    assert actual_files == list_of_expected_files


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_files_to_download_market() -> None:
    """
    Tests the data exporter get list of files function works as expected on a fixed dataframe. This can be
    considered an integration test because it will call the API, however our data schema and data file history
    should stay the same so the dependency is minimal
    """
    expected_file_names = [
        "market-trades-spot/binance/2017-07-15.json.gz",
        "market-trades-spot/binance/2017-07-16.json.gz",
        "market-trades-spot/binance/2017-07-17.json.gz",
    ]
    start_date = datetime(2017, 7, 15)
    end_date = datetime(2017, 7, 17)
    exchange = "binance"
    market_trades_url = "market-trades-spot"
    actual_file_names = data_exporter._get_list_of_file_urls_to_download_market_trades(
        start_date=start_date,
        end_date=end_date,
        exchanges=exchange,
        route_url=market_trades_url,
    )
    assert expected_file_names == actual_file_names


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_files_to_download_market_future() -> None:
    expected_file_names = [
        "market-trades-future/binance/2019-09-15.json.gz",
        "market-trades-future/binance/2019-09-16.json.gz",
        "market-trades-future/binance/2019-09-17.json.gz",
    ]
    start_date = datetime(2019, 9, 15)
    end_date = datetime(2019, 9, 17)
    exchange = "binance"
    market_trades_url = "market-trades-future"
    actual_file_names = data_exporter._get_list_of_file_urls_to_download_market_trades(
        start_date=start_date,
        end_date=end_date,
        exchanges=exchange,
        route_url=market_trades_url,
    )
    assert expected_file_names == actual_file_names


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_files_to_download_market_empty() -> None:
    expected_file_names: List[str] = []
    start_date = datetime(2015, 7, 15)
    end_date = datetime(2015, 7, 17)
    exchange = "binance"
    actual_file_names = data_exporter._get_list_of_file_urls_to_download_market_trades(
        route_url="market-trades-spot",
        start_date=start_date,
        end_date=end_date,
        exchanges=exchange,
    )
    assert expected_file_names == actual_file_names


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_files_market_quotes() -> None:
    expected_file_names: List[str] = []
    start_date = datetime(2015, 7, 15)
    end_date = datetime(2015, 7, 17)
    exchange = "Binance"
    quotes_route = "market-quotes-spot"
    assets = ["1INCHUSD"]
    actual_file_names = data_exporter._get_list_files_to_download_from_ff_server(
        assets_pairs=assets,
        route_url=quotes_route,
        start_date=start_date,
        end_date=end_date,
        exchanges=exchange,
    )
    assert expected_file_names == actual_file_names


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_download_jsongz_market_trades() -> None:
    """
    Tests downloading some market trades spot data
    """
    assert not (os.path.exists("market-trades-spot/binance/2017-07-15.json.gz"))
    assert not (os.path.exists("market-trades-spot/binance/2017-07-16.json.gz"))
    assert not (os.path.exists("market-trades-spot/binance/2017-07-17.json.gz"))
    start_date = datetime(2017, 7, 15)
    end_date = datetime(2017, 7, 17)
    data_exporter.export_market_trades_spot_data(
        start_date=start_date,
        end_date=end_date,
        exchanges="binance",
        output_format="json.gz",
    )
    assert os.path.exists("market-trades-spot/binance/2017-07-15.json.gz")
    assert os.path.exists("market-trades-spot/binance/2017-07-16.json.gz")
    assert os.path.exists("market-trades-spot/binance/2017-07-17.json.gz")


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_download_market_trades_futures() -> None:
    list_of_expected_files = [
        "market-trades-future/binance/2022-07-15.json.gz",
        "market-trades-future/binance/2022-07-16.json.gz",
        "market-trades-future/binance/2022-07-17.json.gz",
    ]
    files_downloaded_test_helper_start(list_of_expected_files)
    start_date = datetime(2022, 7, 15)
    end_date = datetime(2022, 7, 17)
    data_exporter.export_market_trades_future_data(
        start_date=start_date,
        end_date=end_date,
        exchanges="binance",
        output_format="json.gz",
    )
    files_downloaded_test_helper_end(list_of_expected_files)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_download_json_market_trades_spot() -> None:
    list_of_expected_files = [
        "market-trades-spot/binance/2019-07-15.json",
        "market-trades-spot/binance/2019-07-16.json",
        "market-trades-spot/binance/2019-07-17.json",
    ]
    files_downloaded_test_helper_start(list_of_expected_files)
    start_date = datetime(2019, 7, 15)
    end_date = datetime(2019, 7, 17)
    data_exporter.export_market_trades_spot_data(
        start_date=start_date,
        end_date=end_date,
        exchanges="binance",
        output_format="json",
    )
    files_downloaded_test_helper_end(list_of_expected_files)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_download_csv_market_trades() -> None:
    """
    Tests the same set of files are created fine using the csv output option
    """
    assert not (os.path.exists("market-trades-spot/binance/2017-07-15.csv"))
    assert not (os.path.exists("market-trades-spot/binance/2017-07-16.csv"))
    assert not (os.path.exists("market-trades-spot/binance/2017-07-17.csv"))
    start_date = datetime(2017, 7, 15)
    end_date = datetime(2017, 7, 17)
    data_exporter.export_market_trades_spot_data(
        start_date=start_date,
        end_date=end_date,
        exchanges="binance",
        output_format="csv",
    )
    assert os.path.exists("market-trades-spot/binance/2017-07-15.csv")
    assert os.path.exists("market-trades-spot/binance/2017-07-16.csv")
    assert os.path.exists("market-trades-spot/binance/2017-07-17.csv")


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_download_json_market_trades() -> None:
    """
    Tests the same set of files are created as json files
    """
    assert not (os.path.exists("market-trades-spot/binance/2017-07-15.json"))
    assert not (os.path.exists("market-trades-spot/binance/2017-07-16.json"))
    assert not (os.path.exists("market-trades-spot/binance/2017-07-17.json"))
    start_date = datetime(2017, 7, 15)
    end_date = datetime(2017, 7, 17)
    data_exporter.export_market_trades_spot_data(
        start_date=start_date,
        end_date=end_date,
        exchanges="binance",
        output_format="json",
    )
    assert os.path.exists("market-trades-spot/binance/2017-07-15.json")
    assert os.path.exists("market-trades-spot/binance/2017-07-16.json")
    assert os.path.exists("market-trades-spot/binance/2017-07-17.json")
    loaded_data = json.load(open("market-trades-spot/binance/2017-07-15.json"))
    assert (
        loaded_data[0]["market"] == "binance-bnb-btc-spot"
    )  # spot checking a known value in the data file


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_invalid_exchange_value_error() -> None:
    """
    Tests that ValueError is thrown when an invalid exchange is provided
    """
    start_date = datetime(2017, 7, 15)
    end_date = datetime(2017, 7, 17)
    with pytest.raises(ValueError) as excinfo:
        data_exporter.export_market_trades_spot_data(
            start_date=start_date,
            end_date=end_date,
            exchanges=["invalid_exchange", "binance"],
            output_format="csv",
        )


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_invalid_output_format_exchange_value_error() -> None:
    """
    Tests that ValueError is thrown when an invalid output format is provided
    """
    start_date = datetime(2017, 7, 15)
    end_date = datetime(2017, 7, 17)
    with pytest.raises(ValueError) as excinfo:
        data_exporter.export_market_trades_spot_data(
            start_date=start_date,
            end_date=end_date,
            exchanges=["binance"],
            output_format="invalid_format",
        )


if __name__ == "__main__":
    pytest.main()




import pytest

from coinmetrics.api_client import CoinMetricsClient
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")), verbose=True)
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)

@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_trades() -> None:
    catalog_market_trades = client.catalog_market_trades_v2(page_size=10).first_page()
    assert len(catalog_market_trades) == 10
    assert all(['market' in catalog for catalog in catalog_market_trades])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_trades() -> None:
    catalog_market_trades = client.catalog_full_market_trades_v2(page_size=10).first_page()
    assert len(catalog_market_trades) == 10
    assert all(['market' in catalog for catalog in catalog_market_trades])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_candles() -> None:
    catalog_market_candles = client.catalog_full_market_candles_v2(page_size=10).first_page()
    assert len(catalog_market_candles) == 10
    assert all(['market' in catalog for catalog in catalog_market_candles])
    assert all(['frequencies' in catalog for catalog in catalog_market_candles])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_candles() -> None:
    catalog_market_candles = client.catalog_market_candles_v2(page_size=10).first_page()
    assert len(catalog_market_candles) == 10
    assert all(['market' in catalog for catalog in catalog_market_candles])
    assert all(['frequencies' in catalog for catalog in catalog_market_candles])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_contract_prices() -> None:
    catalog_contract_prices = client.catalog_market_contract_prices_v2(page_size=10).first_page()
    assert len(catalog_contract_prices) == 10
    assert all(['market' in catalog for catalog in catalog_contract_prices])
    assert all(['min_time' in catalog for catalog in catalog_contract_prices])
    assert all(['max_time' in catalog for catalog in catalog_contract_prices])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_contract_prices() -> None:
    catalog_contract_prices = client.catalog_full_market_contract_prices_v2(page_size=10).first_page()
    assert len(catalog_contract_prices) == 10
    assert all(['market' in catalog for catalog in catalog_contract_prices])
    assert all(['min_time' in catalog for catalog in catalog_contract_prices])
    assert all(['max_time' in catalog for catalog in catalog_contract_prices])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_funding_rates() -> None:
    catalog_funding_rates = client.catalog_full_market_funding_rates_v2(page_size=10).first_page()
    assert len(catalog_funding_rates) == 10
    assert all(['market' in catalog for catalog in catalog_funding_rates])
    assert all(['min_time' in catalog for catalog in catalog_funding_rates])
    assert all(['max_time' in catalog for catalog in catalog_funding_rates])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_funding_rates() -> None:
    catalog_funding_rates = client.catalog_market_funding_rates_v2(page_size=10).first_page()
    assert len(catalog_funding_rates) == 10
    assert all(['market' in catalog for catalog in catalog_funding_rates])
    assert all(['min_time' in catalog for catalog in catalog_funding_rates])
    assert all(['max_time' in catalog for catalog in catalog_funding_rates])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_greeks() -> None:
    catalog_greeks = client.catalog_market_greeks_v2(page_size=10).first_page()
    assert len(catalog_greeks) == 10
    assert all(['market' in catalog for catalog in catalog_greeks])
    assert all(['min_time' in catalog for catalog in catalog_greeks])
    assert all(['max_time' in catalog for catalog in catalog_greeks])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_greeks() -> None:
    catalog_greeks = client.catalog_full_market_greeks_v2(page_size=10).first_page()
    assert len(catalog_greeks) == 10
    assert all(['market' in catalog for catalog in catalog_greeks])
    assert all(['min_time' in catalog for catalog in catalog_greeks])
    assert all(['max_time' in catalog for catalog in catalog_greeks])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_implied_volatility() -> None:
    catalog_market_implied_vol = client.catalog_full_market_implied_volatility_v2(page_size=10).first_page()
    assert len(catalog_market_implied_vol) == 10
    assert all(['market' in catalog for catalog in catalog_market_implied_vol])
    assert all(['min_time' in catalog for catalog in catalog_market_implied_vol])
    assert all(['max_time' in catalog for catalog in catalog_market_implied_vol])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_implied_volatility() -> None:
    catalog_market_implied_vol = client.catalog_market_implied_volatility_v2(page_size=10).first_page()
    assert len(catalog_market_implied_vol) == 10
    assert all(['market' in catalog for catalog in catalog_market_implied_vol])
    assert all(['min_time' in catalog for catalog in catalog_market_implied_vol])
    assert all(['max_time' in catalog for catalog in catalog_market_implied_vol])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_liquidations() -> None:
    catalog_market_liquidations = client.catalog_market_liquidations_v2(page_size=10).first_page()
    assert len(catalog_market_liquidations) == 10
    assert all(['market' in catalog for catalog in catalog_market_liquidations])
    assert all(['min_time' in catalog for catalog in catalog_market_liquidations])
    assert all(['max_time' in catalog for catalog in catalog_market_liquidations])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_liquidations() -> None:
    catalog_market_liquidations = client.catalog_full_market_liquidations_v2(page_size=10).first_page()
    assert len(catalog_market_liquidations) == 10
    assert all(['market' in catalog for catalog in catalog_market_liquidations])
    assert all(['min_time' in catalog for catalog in catalog_market_liquidations])
    assert all(['max_time' in catalog for catalog in catalog_market_liquidations])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_openinterest() -> None:
    catalog_market_openinterest = client.catalog_full_market_open_interest_v2(page_size=10).first_page()
    assert len(catalog_market_openinterest) == 10
    assert all(['market' in catalog for catalog in catalog_market_openinterest])
    assert all(['min_time' in catalog for catalog in catalog_market_openinterest])
    assert all(['max_time' in catalog for catalog in catalog_market_openinterest])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_openinterest() -> None:
    catalog_market_openinterest = client.catalog_market_open_interest_v2(page_size=10).first_page()
    assert len(catalog_market_openinterest) == 10
    assert all(['market' in catalog for catalog in catalog_market_openinterest])
    assert all(['min_time' in catalog for catalog in catalog_market_openinterest])
    assert all(['max_time' in catalog for catalog in catalog_market_openinterest])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_orderbooks() -> None:
    catalog_market_orderbooks = client.catalog_market_orderbooks_v2(page_size=10).first_page()
    assert len(catalog_market_orderbooks) == 10
    assert all(['market' in catalog for catalog in catalog_market_orderbooks])
    assert all(['min_time' in catalog for catalog in catalog_market_orderbooks])
    assert all(['max_time' in catalog for catalog in catalog_market_orderbooks])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_quotes() -> None:
    catalog_market_quotes = client.catalog_market_quotes_v2(page_size=10).first_page()
    assert len(catalog_market_quotes) == 10
    assert all(['market' in catalog for catalog in catalog_market_quotes])
    assert all(['min_time' in catalog for catalog in catalog_market_quotes])
    assert all(['max_time' in catalog for catalog in catalog_market_quotes])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_quotes() -> None:
    catalog_market_quotes = client.catalog_full_market_quotes_v2(page_size=10).first_page()
    assert len(catalog_market_quotes) == 10
    assert all(['market' in catalog for catalog in catalog_market_quotes])
    assert all(['min_time' in catalog for catalog in catalog_market_quotes])
    assert all(['max_time' in catalog for catalog in catalog_market_quotes])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_orderbooks() -> None:
    catalog_market_orderbooks = client.catalog_full_market_orderbooks_v2(page_size=10).first_page()
    assert len(catalog_market_orderbooks) == 10
    assert all(['market' in catalog for catalog in catalog_market_orderbooks])
    assert all(['min_time' in catalog for catalog in catalog_market_orderbooks])
    assert all(['max_time' in catalog for catalog in catalog_market_orderbooks])

@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_metrics() -> None:
    catalog_market_metrics = client.catalog_full_market_metrics_v2(page_size=10).first_page()
    assert len(catalog_market_metrics) == 10
    assert all(['market' in catalog for catalog in catalog_market_metrics])
    assert all(['metrics' in catalog for catalog in catalog_market_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_metrics() -> None:
    catalog_market_metrics = client.catalog_market_metrics_v2(page_size=10).first_page()
    assert len(catalog_market_metrics) == 10
    assert all(['market' in catalog for catalog in catalog_market_metrics])
    assert all(['metrics' in catalog for catalog in catalog_market_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_asset_metrics() -> None:
    catalog_asset_metrics = client.catalog_asset_metrics_v2(page_size=10).first_page()
    assert len(catalog_asset_metrics) == 10
    assert all(['asset' in catalog for catalog in catalog_asset_metrics])
    assert all(['metrics' in catalog for catalog in catalog_asset_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_asset_metrics() -> None:
    catalog_asset_metrics = client.catalog_full_asset_metrics_v2(page_size=10).first_page()
    assert len(catalog_asset_metrics) == 10
    assert all(['asset' in catalog for catalog in catalog_asset_metrics])
    assert all(['metrics' in catalog for catalog in catalog_asset_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_metrics() -> None:
    catalog_exchange_metrics = client.catalog_exchange_metrics_v2(page_size=10).first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_exchange_metrics() -> None:
    catalog_exchange_metrics = client.catalog_full_exchange_metrics_v2(page_size=10).first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_asset_metrics() -> None:
    catalog_exchange_metrics = client.catalog_exchange_asset_metrics_v2(page_size=10).first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange_asset' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_exchange_asset_metrics() -> None:
    catalog_exchange_metrics = client.catalog_full_exchange_asset_metrics_v2(page_size=10).first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange_asset' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_pair_metrics() -> None:
    catalog_pair_metrics = client.catalog_pair_metrics_v2(page_size=10).first_page()
    assert len(catalog_pair_metrics) == 10
    assert all(['pair' in catalog for catalog in catalog_pair_metrics])
    assert all(['metrics' in catalog for catalog in catalog_pair_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_full_pair_metrics() -> None:
    catalog_pair_metrics = client.catalog_full_pair_metrics_v2(page_size=10).first_page()
    assert len(catalog_pair_metrics) == 10
    assert all(['pair' in catalog for catalog in catalog_pair_metrics])
    assert all(['metrics' in catalog for catalog in catalog_pair_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_institution_pair_metrics() -> None:
    catalog_institution_metrics = client.catalog_institution_metrics_v2(page_size=10).first_page()
    assert all(['institution' in catalog for catalog in catalog_institution_metrics])
    assert all(['metrics' in catalog for catalog in catalog_institution_metrics])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_institution_pair_metrics() -> None:
    catalog_institution_metrics = client.catalog_full_institution_metrics_v2(page_size=10).first_page()
    assert all(['institution' in catalog for catalog in catalog_institution_metrics])
    assert all(['metrics' in catalog for catalog in catalog_institution_metrics])


if __name__ == '__main__':
    pytest.main()


import os
import threading
import pytest

from coinmetrics.api_client import CoinMetricsClient

client = CoinMetricsClient()
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_rate_limiter_works_as_expected() -> None:
    """
    This function tests if a community user can run a 20 second query against our API without tripping the 429 error
    """
    def target() -> None:
        try:
            client.get_asset_metrics(assets="btc,algo,eth,usdc", metrics="ReferenceRateUSD", frequency="1m").to_list()
        except Exception as e:
            pytest.fail(f"Function failed with exception: {e}")

    test_thread = threading.Thread(target=target)
    test_thread.daemon = True
    test_thread.start()

    test_thread.join(20)

    if not test_thread.is_alive():
        pytest.fail("Function failed to run for 20 seconds without throwing an exception")


if __name__ == '__main__':
    pytest.main()


import pytest
import requests
from datetime import datetime
from coinmetrics._exceptions import (
    CoinMetricsClientQueryParamsException,
    CoinMetricsUnauthorizedException,
)
from coinmetrics.api_client import CoinMetricsClient
from coinmetrics.data_exporter import CoinMetricsDataExporter
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")))
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_custom_414_error() -> None:
    """
    This tests that the
    """
    markets = [market["market"] for market in client.catalog_markets()]
    try:
        intentional_414 = client.catalog_markets(markets=markets)
    except Exception as e:
        assert type(e) == CoinMetricsClientQueryParamsException
        print(e)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_custom_exception_not_raised_for_403() -> None:
    """
    Tests that custom exceptions are not raised for unauthorized - just a normal HTTPError, If a different error is
    is raised it asserts false
    """
    unauthorized_client = CoinMetricsClient(api_key="Invalid API Key")
    try:
        intentional_401 = unauthorized_client.catalog_markets()
    except requests.HTTPError as e:
        assert e.response.status_code == 401
    except Exception as e:
        assert False


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_403_error_message() -> None:
    invalid_api_key = "invalid_key"
    data_exporter = CoinMetricsDataExporter(api_key=invalid_api_key)
    try:
        start_date = datetime(2019, 1, 1)
        end_date = datetime(2019, 1, 31)
        exchanges = ["coinbase", "gemini"]
        data_exporter.export_market_trades_spot_data(
            start_date=start_date, end_date=end_date, exchanges=exchanges, threaded=True
        )
    except CoinMetricsUnauthorizedException as e:
        assert e.response.status_code == 403 or e.response.status_code == 401
        print(e)


if __name__ == "__main__":
    pytest.main()
